@@ -950,15 +950,23 @@ class LoadParser : public CatalogParser
         LoadParser(Catalog& c, wxTextFile *f)
               : CatalogParser(f),
                 FileIsValid(false),
-                m_catalog(c), m_nextId(1), m_seenHeaderAlready(false) {}
+                m_catalog(c), m_nextId(1), m_seenHeaderAlready(false), m_collectMsgidText(true) {}
 
         // true if the file is valid, i.e. has at least some data
         bool FileIsValid;
 
         Language GetMsgidLanguage()
         {
-            auto utf8 = m_allMsgidText.utf8_str();
-            return Language::TryDetectFromText(utf8.data(), utf8.length(), Language::English());
+            auto x_srclang = m_catalog.m_header.GetHeader("X-Source-Language");
+            if (!x_srclang.empty())
+            {
+                return Language::TryParse(str::to_utf8(x_srclang));
+            }
+            else
+            {
+                auto utf8 = m_allMsgidText.utf8_str();
+                return Language::TryDetectFromText(utf8.data(), utf8.length(), Language::English());
+            }
         }
 
     protected:
@@ -991,6 +999,7 @@ class LoadParser : public CatalogParser
         bool m_seenHeaderAlready;
 
         // collected text of msgids, with newlines, for language detection
+        bool m_collectMsgidText;
         wxString m_allMsgidText;
 };
 
@@ -1019,6 +1028,7 @@ bool LoadParser::OnEntry(const wxString& msgid,
             // gettext header:
             m_catalog.m_header.FromString(mtranslations[0]);
             m_catalog.m_header.Comment = comment;
+            m_collectMsgidText = m_catalog.m_header.GetHeader("X-Source-Language").empty();
             m_seenHeaderAlready = true;
         }
         // else: ignore duplicate header in malformed files
@@ -1054,12 +1064,15 @@ bool LoadParser::OnEntry(const wxString& msgid,
         m_catalog.AddItem(d);
 
         // collect text for language detection:
-        m_allMsgidText.append(msgid);
-        m_allMsgidText.append('\n');
-        if (!msgid_plural.empty())
+        if (m_collectMsgidText)
         {
-            m_allMsgidText.append(msgid_plural);
+            m_allMsgidText.append(msgid);
             m_allMsgidText.append('\n');
+            if (!msgid_plural.empty())
+            {
+                m_allMsgidText.append(msgid_plural);
+                m_allMsgidText.append('\n');
+            }
         }
     }
     return true;
