@@ -308,7 +308,9 @@ bool PoeditApp::CheckForBetaUpdates() const
 }
 
 
+#ifndef __WXOSX__
 static wxArrayString gs_filesToOpen;
+#endif
 
 extern void InitXmlResource();
 
@@ -406,6 +408,12 @@ bool PoeditApp::OnInit()
     {
         wxRenameFile(oldcfgfile, CFG_FILE);
     }
+
+    // Remove legacy Sparkle updates folder that used to accumulate broken download
+    // files for some users and eat up disk space:
+    auto oldsparkle = wxStandardPaths::Get().GetUserDataDir() + "/.Sparkle";
+    if (wxFileName::DirExists(oldsparkle))
+        wxFileName::Rmdir(oldsparkle, wxPATH_RMDIR_RECURSIVE);
 #endif
 
     wxConfigBase::Set(
@@ -464,6 +472,7 @@ bool PoeditApp::OnInit()
     AssociateFileTypeIfNeeded();
 #endif
 
+#ifndef __WXOSX__
     // NB: opening files or creating empty window is handled differently on
     //     Macs, using MacOpenFiles() and MacNewFile(), so don't create empty
     //     window if no files are given on command line; but still support
@@ -473,7 +482,6 @@ bool PoeditApp::OnInit()
         OpenFiles(gs_filesToOpen);
         gs_filesToOpen.clear();
     }
-#ifndef __WXOSX__
     else
     {
         OpenNewFile();
@@ -701,7 +709,7 @@ void PoeditApp::SetDefaultExtractors(wxConfigBase *cfg)
         { 1, "YCP",        "*.ycp" },
         { 1, "awk",        "*.awk" },
         { 1, "librep",     "*.jl" },
-        { NULL, NULL }
+        { 0, NULL, NULL }
     };
 
     for (size_t i = 0; s_gettextLangs[i].name != NULL; i++)
@@ -867,8 +875,16 @@ bool PoeditApp::OnCmdLineParsed(wxCmdLineParser& parser)
         CallAfter([=]{ HandleCustomURI(poeditURI); });
     }
 
+#ifdef __WXOSX__
+    wxArrayString filesToOpen;
+    for (size_t i = 0; i < parser.GetParamCount(); i++)
+        filesToOpen.Add(parser.GetParam(i));
+    if (!filesToOpen.empty())
+        OSXStoreOpenFiles(filesToOpen);
+#else
     for (size_t i = 0; i < parser.GetParamCount(); i++)
         gs_filesToOpen.Add(parser.GetParam(i));
+#endif
 
     return true;
 }
