@@ -54,22 +54,14 @@ namespace
 // is what GTK+ uses in its tree view control)
 #define DARKEN_FACTOR      0.95
 
-// max difference in color to consider it "amost" same if it differs by most
-// this amount (in 0..255 range) from the tested color:
-#define COLOR_SIMILARITY_FACTOR  20
-
 inline bool IsAlmostBlack(const wxColour& clr)
 {
-    return (clr.Red() <= COLOR_SIMILARITY_FACTOR &&
-            clr.Green() <= COLOR_SIMILARITY_FACTOR &&
-            clr.Blue() <= COLOR_SIMILARITY_FACTOR);
+    return clr.Red() < 0x60 && clr.Green() < 0x60 && clr.Blue() < 0x60;
 }
 
 inline bool IsAlmostWhite(const wxColour& clr)
 {
-    return (clr.Red() >= 255-COLOR_SIMILARITY_FACTOR &&
-            clr.Green() >= 255-COLOR_SIMILARITY_FACTOR &&
-            clr.Blue() >= 255-COLOR_SIMILARITY_FACTOR);
+    return clr.Red() > 0xE0 && clr.Green() > 0xE0 && clr.Blue() > 0xE0;
 }
 
 
@@ -162,8 +154,8 @@ PoeditListCtrl::PoeditListCtrl(wxWindow *parent,
 
     m_displayIDs = dispIDs;
 
-    m_isRTL = false;
-    m_appIsRTL = (wxTheApp->GetLayoutDirection() == wxLayout_RightToLeft);
+    m_sourceTextDir = m_transTextDir = TextDirection::LTR;
+    m_appTextDir = (wxTheApp->GetLayoutDirection() == wxLayout_RightToLeft) ? TextDirection::RTL : TextDirection::LTR;
 
     sortOrder = SortOrder::Default();
 
@@ -328,7 +320,7 @@ void PoeditListCtrl::CreateColumns()
         m_colId = -1;
 
 #ifdef __WXMSW__
-    if (m_appIsRTL)
+    if (m_appTextDir == TextDirection::RTL)
     {
         // another wx quirk: if we truly need left alignment, we must lie under RTL locales
         wxListItem colInfoOrig;
@@ -406,13 +398,15 @@ void PoeditListCtrl::ReadCatalog(bool resetSizeAndSelection)
 
     auto srclang = m_catalog->GetSourceLanguage();
     auto lang = m_catalog->GetLanguage();
+    m_sourceTextDir = srclang.Direction();
+    m_transTextDir = lang.Direction();
+
     auto isRTL = lang.IsRTL();
 #ifdef __WXMSW__
     // a quirk of wx API: if the current locale is RTL, the meaning of L and R is reversed
-    if (m_appIsRTL)
+    if (m_appTextDir == TextDirection::RTL)
         isRTL = !isRTL;
 #endif
-    m_isRTL = isRTL;
 
     wxListItem colInfo;
     colInfo.SetMask(wxLIST_MASK_TEXT);
@@ -513,20 +507,26 @@ wxString PoeditListCtrl::OnGetItemText(long item, long column) const
             orig = d->GetString();
 
         orig = orig.substr(0, GetMaxColChars());
+#ifdef __WXMSW__
+        orig.Replace("\n", " ");
+#endif
 
         // Add RTL Unicode mark to render bidi texts correctly
-        if (m_appIsRTL)
-            return bidi::LRE + orig;
+        if (m_appTextDir != m_sourceTextDir)
+            return bidi::mark_direction(orig, m_sourceTextDir);
         else
             return orig;
     }
     else if (column == m_colTrans)
     {
         auto trans = d->GetTranslation().substr(0, GetMaxColChars());
+#ifdef __WXMSW__
+        trans.Replace("\n", " ");
+#endif
 
         // Add RTL Unicode mark to render bidi texts correctly
-        if (m_isRTL && !m_appIsRTL)
-            return bidi::RLE + trans;
+        if (m_appTextDir != m_transTextDir)
+            return bidi::mark_direction(trans, m_transTextDir);
         else
             return trans;
     }
