@@ -1,5 +1,5 @@
 ﻿/*
- *  This file is part of Poedit (http://poedit.net)
+ *  This file is part of Poedit (https://poedit.net)
  *
  *  Copyright (C) 2003 Christophe Hermier
  *  Copyright (C) 2013-2016 Vaclav Slavik
@@ -121,10 +121,10 @@ void Catalog::ExportToHTML(std::ostream& f)
 
     std::string lang_src, lang_tra;
     if (m_sourceLanguage.IsValid())
-        lang_src = " lang='" + m_sourceLanguage.RFC3066() + "'";
+        lang_src = " lang='" + m_sourceLanguage.LanguageTag() + "'";
     if (m_header.Lang.IsValid())
     {
-        lang_tra = " lang='" + m_header.Lang.RFC3066() + "'";
+        lang_tra = " lang='" + m_header.Lang.LanguageTag() + "'";
         if (m_header.Lang.IsRTL())
             lang_tra += " dir='rtl'";
     }
