@@ -1,5 +1,5 @@
 ﻿/*
- *  This file is part of Poedit (http://poedit.net)
+ *  This file is part of Poedit (https://poedit.net)
  *
  *  Copyright (C) 2015-2016 Vaclav Slavik
  *
@@ -80,6 +80,6 @@ inline double HiDPIScalingFactor() { return 1.0; }
 
     Note that @a name is given *without* the ".png" extension.
  */
-extern wxBitmap LoadScaledBitmap(const wxString& name);
+extern wxBitmap LoadScaledBitmap(const wxString& name, bool mirror);
 
 #endif // Poedit_hidpi_h
