@@ -1,5 +1,5 @@
 /*
- *  This file is part of Poedit (http://poedit.net)
+ *  This file is part of Poedit (https://poedit.net)
  *
  *  Copyright (C) 2013-2016 Vaclav Slavik
  *
@@ -31,6 +31,14 @@
 #include <vector>
 #include <unicode/locid.h>
 
+/// Language's text writing direction
+enum class TextDirection
+{
+    LTR,
+    RTL
+};
+
+
 /// Representation of translation's language.
 class Language
 {
@@ -50,11 +58,11 @@ public:
     /// Returns optional variant (after @, e.g. 'latin', typically empty)
     std::string Variant() const;
 
-    /// Return code formatted as in RFC 3066, e.g. en-US
-    std::string RFC3066() const;
+    /// Return language tag for the language, per BCP 47, e.g. en-US or sr-Latn
+    std::string LanguageTag() const { return m_tag; }
 
     /// Returns name of the locale suitable for ICU
-    std::string IcuLocaleName() const { return LangAndCountry(); }
+    std::string IcuLocaleName() const { return LanguageTag(); }
     /// Returns ICU equivalent of the language info
     icu::Locale ToIcu() const;
 
@@ -90,8 +98,11 @@ public:
      */
     std::string DefaultPluralFormsExpr() const;
 
+    /// Returns language's text writing direction
+    TextDirection Direction() const { return m_direction; }
+
     /// Returns true if the language is written right-to-left.
-    bool IsRTL() const;
+    bool IsRTL() const { return m_direction == TextDirection::RTL; }
 
     /**
         Tries to parse the string as language identification.
@@ -153,12 +164,15 @@ public:
     bool operator!=(const Language& other) const { return m_code != other.m_code; }
     bool operator<(const Language& other) const { return m_code < other.m_code; }
 
-protected:
-    Language(const std::string& code) : m_code(code) {}
-    Language(const std::wstring& code) : m_code(code.begin(), code.end()) {}
+private:
+    Language(const std::string& code) { Init(code); }
+    Language(const std::wstring& code) { Init(std::string(code.begin(), code.end())); }
+    void Init(const std::string& code);
 
 private:
     std::string m_code;
+    std::string m_tag;
+    TextDirection m_direction;
 };
 
 #endif // Poedit_language_h
