@@ -1,5 +1,5 @@
 /*
- *  This file is part of Poedit (http://poedit.net)
+ *  This file is part of Poedit (https://poedit.net)
  *
  *  Copyright (C) 2000-2016 Vaclav Slavik
  *
@@ -57,6 +57,7 @@
 #include "hidpi.h"
 #include "language.h"
 #include "str_helpers.h"
+#include "unicode_helpers.h"
 #include "pluralforms/pl_evaluate.h"
 #include "utility.h"
 
@@ -75,7 +76,19 @@ inline wxString RelativePath(const wxString& fn, const wxString& to, wxPathForma
     if (fn == to || fn + wxFILE_SEP_PATH == to)
         return ".";
     auto f = MakeFileName(fn);
-    f.MakeRelativeTo(to);
+    if (!f.MakeRelativeTo(to))
+    {
+#ifdef __WXMSW__
+        if (format == wxPATH_UNIX)
+        {
+            // Paths on different volumes, which are ignored in UNIX path formatting.
+            // The best we can do is to use Windows path with / instead of \ ...
+            wxString dos = f.GetFullPath(wxPATH_DOS);
+            dos.Replace("\\", "/");
+            return dos;
+        }
+#endif
+    }
     return f.GetFullPath(format);
 }
 
@@ -229,7 +242,7 @@ public:
 
     void SetPath(const wxString& path)
     {
-        SetLabel(path);
+        SetLabel(bidi::platform_mark_direction(path));
     }
 };
 
@@ -298,10 +311,12 @@ public:
         m_list->Clear();
         for (auto& p: Array())
         {
+            wxString s;
             if (wxIsWild(p))
-                m_list->Append(p);
+                s = p;
             else
-                m_list->Append(RelativePath(p, m_data->basepath, wxPATH_NATIVE));
+                s = RelativePath(p, m_data->basepath, wxPATH_NATIVE);
+            m_list->Append(bidi::platform_mark_direction(s));
         }
     }
 
@@ -600,12 +615,15 @@ wxString GetCharsetFromCombobox(wxComboBox *ctrl)
     return c;
 }
 
-void GetKeywordsFromControl(wxEditableListBox *box, wxArrayString& output)
+void GetKeywordsFromControl(wxEditableListBox *box, bool defaultKeywords, wxArrayString& output)
 {
     wxArrayString arr;
     box->GetStrings(arr);
 
     output.clear();
+    if (!defaultKeywords)
+        output.push_back("");
+
     for (auto x: arr)
     {
         if (x.empty())
@@ -641,14 +659,19 @@ void PropertiesDialog::TransferTo(const CatalogPtr& cat)
         if (pf_cat == "nplurals=INTEGER; plural=EXPRESSION;")
             pf_cat = pf_def;
 
-        m_pluralFormsExpr->SetValue(pf_cat);
+        m_pluralFormsExpr->SetValue(bidi::mark_direction(pf_cat, TextDirection::LTR));
         if (!pf_cat.empty() && pf_cat == pf_def)
             m_pluralFormsDefault->SetValue(true);
         else
             m_pluralFormsCustom->SetValue(true);
     }
 
-    m_keywords->SetStrings(cat->Header().Keywords);
+    auto kw = cat->Header().Keywords;
+    auto empty_kw = kw.Index("");
+    m_defaultKeywords = (empty_kw == wxNOT_FOUND);
+    if (empty_kw != wxNOT_FOUND)
+        kw.RemoveAt(empty_kw);
+    m_keywords->SetStrings(kw);
 
     m_pathsData->GetFromCatalog(cat);
     m_pathsData->RefreshView();
@@ -680,14 +703,14 @@ void PropertiesDialog::TransferFrom(const CatalogPtr& cat)
 
         if (pluralForms.empty())
         {
-            pluralForms = m_pluralFormsExpr->GetValue().Strip(wxString::both);
+            pluralForms = bidi::strip_control_chars(m_pluralFormsExpr->GetValue().Strip(wxString::both));
             if ( !pluralForms.empty() && !pluralForms.EndsWith(";") )
                 pluralForms += ";";
         }
         cat->Header().SetHeaderNotEmpty("Plural-Forms", pluralForms);
     }
 
-    GetKeywordsFromControl(m_keywords, cat->Header().Keywords);
+    GetKeywordsFromControl(m_keywords, m_defaultKeywords, cat->Header().Keywords);
 
     if (m_pathsData->Changed)
         m_pathsData->SetToCatalog(cat);
@@ -746,7 +769,7 @@ void PropertiesDialog::OnPluralFormsDefault(wxCommandEvent& event)
     {
         wxString defaultForm = lang.DefaultPluralFormsExpr();
         if (!defaultForm.empty())
-            m_pluralFormsExpr->SetValue(defaultForm);
+            m_pluralFormsExpr->SetValue(bidi::mark_direction(defaultForm, TextDirection::LTR));
     }
 
     event.Skip();
@@ -771,7 +794,7 @@ bool PropertiesDialog::Validate()
         m_validatedPlural = 1;
         if (m_pluralFormsCustom->GetValue())
         {
-            wxString form = m_pluralFormsExpr->GetValue();
+            wxString form = bidi::strip_control_chars(m_pluralFormsExpr->GetValue());
             if (!form.empty())
             {
                 std::unique_ptr<PluralFormsCalculator> calc(PluralFormsCalculator::make(form.ToAscii()));
