@@ -1,5 +1,5 @@
 ﻿/*
- *  This file is part of Poedit (http://poedit.net)
+ *  This file is part of Poedit (https://poedit.net)
  *
  *  Copyright (C) 2014-2016 Vaclav Slavik
  *
@@ -31,6 +31,8 @@
 #include "concurrency.h"
 #include "errors.h"
 #include "hidpi.h"
+#include "utility.h"
+#include "unicode_helpers.h"
 
 #include "tm/suggestions.h"
 #include "tm/transmem.h"
@@ -96,7 +98,7 @@ SidebarBlock::SidebarBlock(Sidebar *parent, const wxString& label, int flags)
                          wxSizerFlags().Expand().Border(wxBOTTOM|wxLEFT, PX(2)));
         }
         m_headerSizer = new wxBoxSizer(wxHORIZONTAL);
-        m_headerSizer->Add(new HeadingLabel(parent, label), wxSizerFlags().Expand().Center());
+        m_headerSizer->Add(new HeadingLabel(parent, label), wxSizerFlags().Expand());
         m_sizer->Add(m_headerSizer, wxSizerFlags().Expand().PXDoubleBorder(wxLEFT|wxRIGHT));
     }
     m_innerSizer = new wxBoxSizer(wxVERTICAL);
@@ -206,7 +208,6 @@ public:
     {
         auto text = CommentDialog::RemoveStartHash(item->GetComment());
         text.Trim();
-        m_comment->SetLanguage(m_parent->GetCurrentLanguage());
         m_comment->SetAndWrapLabel(text);
     }
 
@@ -261,8 +262,6 @@ class SuggestionWidget : public wxPanel
 public:
     SuggestionWidget(wxWindow *parent) : wxPanel(parent, wxID_ANY)
     {
-        m_appIsRTL = (wxTheApp->GetLayoutDirection() == wxLayout_RightToLeft);
-
         m_icon = new wxStaticBitmap(this, wxID_ANY, wxNullBitmap);
         m_text = new AutoWrappingText(this, "TEXT");
         m_info = new InfoStaticText(this);
@@ -289,7 +288,7 @@ public:
         }
     }
 
-    void SetValue(int index, const Suggestion& s, Language lang, bool isRTL, const wxBitmap& icon, const wxString& tooltip)
+    void SetValue(int index, const Suggestion& s, Language lang, const wxBitmap& icon, const wxString& tooltip)
     {
         m_value = s;
 
@@ -313,19 +312,8 @@ public:
 
         m_icon->SetBitmap(icon);
 
-        auto text = wxControl::EscapeMnemonics(s.text);
-        if (isRTL)
-            text = L"\u202b" + text;
-        else
-            text = L"\u202a" + text;
-        m_text->SetAndWrapLabel(text);
+        auto text = wxControl::EscapeMnemonics(bidi::mark_direction(s.text, lang));
 
-        // a quirk of wx API: if the current locale is RTL, the meaning of L and R is reversed
-        // for alignments
-        if (m_appIsRTL)
-            isRTL = !isRTL;
-
-        m_text->SetAlignment(isRTL ? wxALIGN_RIGHT : wxALIGN_LEFT);
         m_text->SetLanguage(lang);
         m_text->SetAndWrapLabel(text);
 
@@ -386,7 +374,6 @@ private:
     AutoWrappingText *m_text;
     wxStaticText *m_info;
     wxColour m_bg, m_bgHighlight;
-    bool m_appIsRTL;
 };
 
 
@@ -505,11 +492,10 @@ void SuggestionsSidebarBlock::UpdateSuggestions(const SuggestionsList& hits)
 
     // update shown suggestions:
     auto lang = m_parent->GetCurrentLanguage();
-    bool isRTL = lang.IsRTL();
     for (size_t i = 0; i < m_suggestions.size(); ++i)
     {
         auto s = m_suggestions[i];
-        m_suggestionsWidgets[i]->SetValue((int)i, s, lang, isRTL, GetIconForSuggestion(s), GetTooltipForSuggestion(s));
+        m_suggestionsWidgets[i]->SetValue((int)i, s, lang, GetIconForSuggestion(s), GetTooltipForSuggestion(s));
     }
 
     m_innerSizer->Layout();
@@ -560,12 +546,17 @@ void SuggestionsSidebarBlock::UpdateSuggestionsMenu()
         if (index >= SUGGESTIONS_MENU_ENTRIES)
             break;
 
-        auto text = wxString::Format(formatMask, s.text, index+1);
+        wxString text = s.text;
+        text.Replace("\t", " ");
+        text.Replace("\n", " ");
+        if (text.length() > 100)
+            text = text.substr(0, 100) + L"…";
 
         auto item = m_suggestionMenuItems[index];
         m_suggestionsMenu->Append(item);
 
-        item->SetItemLabel(text);
+        auto label = wxControl::EscapeMnemonics(wxString::Format(formatMask, text, index+1));
+        item->SetItemLabel(label);
         item->SetBitmap(GetIconForSuggestion(s));
 
         index++;
@@ -733,11 +724,16 @@ void SuggestionsSidebarBlock::QueryProvider(SuggestionsBackend& backend, const C
 
 
 Sidebar::Sidebar(wxWindow *parent, wxMenu *suggestionsMenu)
-    : wxPanel(parent, wxID_ANY),
+    : wxPanel(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxNO_BORDER | DoubleBufferingWindowStyle()),
       m_catalog(nullptr),
       m_selectedItem(nullptr)
 {
     SetBackgroundColour(SIDEBAR_BACKGROUND);
+#ifdef __WXMSW__
+    if (!IsWindowsXP())
+        SetDoubleBuffered(true);
+#endif
+
     Bind(wxEVT_PAINT, &Sidebar::OnPaint, this);
 #ifdef __WXOSX__
     SetWindowVariant(wxWINDOW_VARIANT_SMALL);
